import os
import sys
from fnmatch import fnmatchcase
from distutils.util import convert_path
from setuptools import setup

with open('README.rst') as f:
    LONG_DESCRIPTION = f.read()

INSTALL_REQUIRES = ['setuptools']
with open('requirements.txt') as f:
    for line in f:
        stripped = line.strip()
        if not stripped:
            continue
        INSTALL_REQUIRES.append(stripped)

PACKAGES = [
    'ab',
    'accounts',
    'admin_ext',
    'clutch',
    'clutchrpc',
    'clutchtunnel',
    'dashboard',
    'django_ext',
    'stats',
]

SCRIPTS = [
    'bin/clutch-web',
    'bin/clutch-tunnel',
    'bin/clutch-rpc',
    'bin/clutch-all',
    'bin/clutch-config',
    'bin/clutch-test',
]

# Provided as an attribute, so you can append to these instead
# of replicating them:
standard_exclude = ('*.py', '*.pyc', '*$py.class', '*~', '.*', '*.bak')
standard_exclude_directories = ('.*', 'CVS', '_darcs', './build',
                                './dist', 'EGG-INFO', '*.egg-info', './docs')


# (c) 2005 Ian Bicking and contributors; written for Paste (http://pythonpaste.org)
# Licensed under the MIT license: http://www.opensource.org/licenses/mit-license.php
# Note: you may want to copy this into your setup.py file verbatim, as
# you can't import this from another package, when you don't know if
# that package is installed yet.
def find_package_data(
    where='.', package='',
    exclude=standard_exclude,
    exclude_directories=standard_exclude_directories,
    only_in_packages=True,
    show_ignored=False):
    """
    Return a dictionary suitable for use in ``package_data``
    in a distutils ``setup.py`` file.

    The dictionary looks like::

        {'package': [files]}

    Where ``files`` is a list of all the files in that package that
    don't match anything in ``exclude``.

    If ``only_in_packages`` is true, then top-level directories that
    are not packages won't be included (but directories under packages
    will).

    Directories matching any pattern in ``exclude_directories`` will
    be ignored; by default directories with leading ``.``, ``CVS``,
    and ``_darcs`` will be ignored.

    If ``show_ignored`` is true, then all the files that aren't
    included in package data are shown on stderr (for debugging
    purposes).

    Note patterns use wildcards, or can be exact paths (including
    leading ``./``), and all searching is case-insensitive.
    """

    out = {}
    stack = [(convert_path(where), '', package, only_in_packages)]
    while stack:
        where, prefix, package, only_in_packages = stack.pop(0)
        for name in os.listdir(where):
            fn = os.path.join(where, name)
            if os.path.isdir(fn):
                bad_name = False
                for pattern in exclude_directories:
                    if (fnmatchcase(name, pattern)
                        or fn.lower() == pattern.lower()):
                        bad_name = True
                        if show_ignored:
                            print >> sys.stderr, (
                                "Directory %s ignored by pattern %s"
                                % (fn, pattern))
                        break
                if bad_name:
                    continue
                if (os.path.isfile(os.path.join(fn, '__init__.py'))
                    and not prefix):
                    if not package:
                        new_package = name
                    else:
                        new_package = package + '.' + name
                    stack.append((fn, '', new_package, False))
                else:
                    stack.append((fn, prefix + name + '/', package, only_in_packages))
            elif package or not only_in_packages:
                # is a file
                bad_name = False
                for pattern in exclude:
                    if (fnmatchcase(name, pattern)
                        or fn.lower() == pattern.lower()):
                        bad_name = True
                        if show_ignored:
                            print >> sys.stderr, (
                                "File %s ignored by pattern %s"
                                % (fn, pattern))
                        break
                if bad_name:
                    continue
                out.setdefault(package, []).append(prefix + name)
    return out

setup(
    name='clutchserver',
    version='0.1.0',
    description='The server component for Clutch framework and Cluth A/B Testing.',
    long_description=LONG_DESCRIPTION,
    author='Eric Florenzano',
    author_email='eflorenzano@twitter.com',
    packages=PACKAGES,
    package_data=find_package_data(only_in_packages=False),
    zip_safe=False,
    install_requires=INSTALL_REQUIRES,
    scripts=SCRIPTS,
)
